#
#  Name:    MpWxCa.py
#
#  Purpose: WxPython widgets for EPICS PVs controlled via MX.
#
#  Author:  William Lavender
#
#-----------------------------------------------------------------------
#
#  Copyright 2011-2012 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

import time
import wx
import Mp
import MpCa

#
#=============================================================================
#
# We use wx.PostEvent to send value updates from the thread in which a callback
# is invoked to the main thread for GUI processing.  This is based on the
# first example at http://wiki.wxpython.org/LongRunningTasks
#

EVT_UPDATE_ID = wx.NewId()

def EVT_UPDATE( widget, function ):
	widget.Connect( -1, -1, EVT_UPDATE_ID, function )

class UpdateEvent(wx.PyEvent):
	def __init__( self, args ):
		wx.PyEvent.__init__(self)
		self.SetEventType( EVT_UPDATE_ID )
		self.args = args

#
#=============================================================================
#

class Value(wx.StaticText):

	def __init__( self, parent, pv_name, \
			function=None, args=None, \
			id=-1, pos=wx.DefaultPosition, size=wx.DefaultSize, \
			style=0, name=wx.StaticTextNameStr, base=None ):

		wx.StaticText.__init__( self, parent, id=id, \
			label="-----", \
			pos=pos, size=size, style=style, name=name )

		if ( function == None ):
			function = _Value_update

		if ( ( base != 10 ) and ( base != 16 ) \
		    and ( base != 8 ) and ( base != None ) ):
			error_message = \
				"Only base 8, 10, and 16 are supported."

			raise ValueError, error_message

		self.base = base

		self.pv = MpCa.PV( pv_name )

		EVT_UPDATE( self, self.OnUpdate )

		# Arrange for automatic updates of the value.

		args = ( self.pv, self )

		self.callback = \
			self.pv.add_callback( MpCa.DBE_VALUE, function, args )

		# Test for the existence of the PV.

		try:
			value = self.pv.caget()

			function( self.callback, args )

		except Mp.Not_Found_Error:
			self.SetValue( "NOT FOUND" )
			self.Enable(False)
			return

		MpCa.poll()

	def OnUpdate( self, event ):

		value = event.args

		# print "OnUpdate: value =", value

		self.SetLabel( value )

		self.SetSize( self.GetBestSize() )

#----

def _Value_update( callback, args ):

	( pv, widget ) = args

	value = pv.get_local()

	if isinstance( value, list ):
		if ( len(value) == 1 ):
			value = value[0]

	if ( widget.base == 10 ):
		value = "%d" % value
	elif ( widget.base == 16 ):
		value = "%#x" % value
	elif ( widget.base == 8 ):
		value = "%#o" % value
	else:
		value = str( value )

	wx.PostEvent( widget, UpdateEvent(value) )

#
#=============================================================================
#

class ValueEntry(wx.TextCtrl):

	def __init__( self, parent, pv_name, \
			function=None, args=None, \
			id=-1, pos=wx.DefaultPosition, size=wx.DefaultSize, \
			style=0, name=wx.TextCtrlNameStr, \
			validator=wx.DefaultValidator ):

		# Adding wx.TE_PROCESS_ENTER to the style causes the
		# widget to generate wx.EVT_TEXT_ENTER events when
		# the Enter key is pressed.

		style = style | wx.TE_PROCESS_ENTER

		wx.TextCtrl.__init__( self, parent, id=id, \
			value=wx.EmptyString, pos=pos, size=size, \
			style=style, validator=validator )

		self.pv = MpCa.PV( pv_name )

		EVT_UPDATE( self, self.OnUpdate )

		# Arrange for automatic updates of the value.

		if ( function == None ):
			function = _ValueEntry_update

		args = ( self.pv, self )

		self.callback = \
			self.pv.add_callback( MpCa.DBE_VALUE, function, args )

		# Test for the existence of the PV.

		try:
			test = self.pv.caget()

		except Mp.Not_Found_Error:
			self.SetValue( "NOT FOUND" )
			self.Enable(False)
			return

		# Disable the widget if the PV is read only.

		read_only = False

		if ( read_only ):
			self.Enable(False)

		MpCa.poll()

		# Typing keys in the entry field will change the color.

		self.Bind( wx.EVT_TEXT, self.OnText )

		# <Enter> keystrokes will change the value of the PV.

		self.Bind( wx.EVT_TEXT_ENTER, self.OnEnter )

	def OnText( self, event ):
		self.SetBackgroundColour( "yellow" )

	def OnEnter( self, event ):

		value = self.GetValue().strip()

		self.pv.caput( value, wait=False )

		time.sleep(0.5)

		self.SetBackgroundColour( wx.NullColour )

	def OnUpdate( self, event ):

		value = event.args

		# print "ValueEntry.OnUpdate: value =", value

		self.SetValue( value )

		self.SetBackgroundColour( wx.NullColour )

		self.SetSize( self.GetBestSize() )

#----

def _ValueEntry_update( callback, args ):

	( pv, widget ) = args

	value = pv.get_local()

	if isinstance( value, list ):
		if ( len(value) == 1 ):
			value = value[0]

	value = str( value )

	wx.PostEvent( widget, UpdateEvent(value) )

